package com.macro.mall.tiny.modules.business.controller;


import com.macro.mall.tiny.common.api.CommonResult;
import com.macro.mall.tiny.modules.business.dto.CommonIdParam;
import com.macro.mall.tiny.modules.business.dto.SensorQueryParam;
import com.macro.mall.tiny.modules.business.model.KrBSensor;
import com.macro.mall.tiny.modules.business.service.KrBSensorService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.websocket.server.PathParam;
import java.util.List;

/**
 * <p>
 *  前端控制器
 * </p>
 *
 * @author macro
 * @since 2021-10-20
 */
@RestController
@Api(tags = "KrBSensorController", description = "提供传感器相关API")
@RequestMapping("/device")
public class KrBSensorController {

    @Autowired
    private KrBSensorService sensorService;

    @ApiOperation("根据线路名称或传感器mac获取传感器列表")
    @PostMapping("/get")
    public CommonResult<List<KrBSensor>> get(@RequestBody SensorQueryParam sensorQueryParam) {
        String lineName = sensorQueryParam.getLineName();
        String deviceMac = sensorQueryParam.getDeviceMac();
        List<KrBSensor> sensorList = sensorService.get(lineName, deviceMac);
        return CommonResult.success(sensorList);
    }

    @ApiOperation("新增传感器")
    @PostMapping("/add")
    public CommonResult add(@RequestBody KrBSensor sensor) {
        boolean success = sensorService.save(sensor);
        if (success) {
            return CommonResult.success(null);
        }
        return CommonResult.failed();
    }

    @ApiOperation("修改传感器")
    @PostMapping("/update")
    public CommonResult update(@RequestBody KrBSensor sensor) {
        boolean success = sensorService.updateById(sensor);
        if (success) {
            return CommonResult.success(null);
        }
        return CommonResult.failed();
    }

    @ApiOperation("删除传感器")
    @PostMapping("/delete")
    public CommonResult delete(@RequestBody CommonIdParam idParam) {
        Long id = idParam.getId();
        boolean success = sensorService.removeById(id);
        if (success) {
            return CommonResult.success(null);
        }
        return CommonResult.failed();
    }

    @ApiOperation("按照线路ID查询传感器列表列表")
    @RequestMapping(value = "/findByLine", method = RequestMethod.GET)
    @ResponseBody
    public CommonResult<List<KrBSensor>> findSensorByLine(@RequestParam @PathParam(value = "lineName") String lineName) {
        List<KrBSensor> sensors = sensorService.findSensorByLine(lineName);
        return CommonResult.success(sensors);
    }

//    @ApiOperation("按照线路ID开启传感器监听")
//    @RequestMapping(value = "/openListenByLineId", method = RequestMethod.GET)
//    @ResponseBody
//    public CommonResult<String> openListenByLine(@RequestParam @PathParam(value = "lineId") Long lineId) {
//        boolean result = sensorService.openListenByLine(lineId);
//        return result ? CommonResult.success("开启监听成功") : CommonResult.failed("开启监听失败");
//    }

//    @ApiOperation("按照线路ID关闭传感器监听")
//    @RequestMapping(value = "/closeListenByLineId", method = RequestMethod.GET)
//    @ResponseBody
//    public CommonResult<String> closeListenByLine(@RequestParam @PathParam(value = "lineId") Long lineId) {
//        boolean result = sensorService.closeListenByLine(lineId);
//        return result ? CommonResult.success("关闭监听成功") : CommonResult.failed("关闭监听失败");
//    }
}

