package com.macro.mall.tiny.modules.business.service;

import cn.hutool.json.JSONUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.data.redis.connection.Message;
import org.springframework.data.redis.connection.MessageListener;
import org.springframework.stereotype.Component;

import javax.websocket.Session;
import java.io.IOException;
import java.util.HashSet;
import java.util.Set;

/**
 * redis消息订阅监听者
 */
@Component
@Slf4j
public class RedisSubscribeListener implements MessageListener {


    //webSocket客户端会话对象
    private Session session;

    private Set<String> sensorMacSet = new HashSet<>();

    /**
     * 接收发布者消息
     *
     * @param message
     * @param bytes
     */
    @Override
    public void onMessage(Message message, byte[] bytes) {
        String msg = new String(message.getBody());
        log.debug("[{}]主题发布：{}", new String(bytes), msg);
        String deviceMac = JSONUtil.parseObj(msg).getStr("deviceMac");
        if (session != null && session.isOpen()) {
            if (!sensorMacSet.contains(deviceMac)) {
                return;
            }
            try {
                session.getBasicRemote().sendText(msg);
            } catch (IOException e) {
                log.error("[redis监听器]发布消息异常：", e);
            }
        }
    }

    public Session getSession() {
        return session;
    }

    public void setSession(Session session) {
        this.session = session;
    }

    public Set<String> getSensorMacSet() {
        return sensorMacSet;
    }

    public void setSensorMacSet(Set<String> sensorMacSet) {
        this.sensorMacSet = sensorMacSet;
    }
}