package com.macro.mall.tiny.modules.business.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.macro.mall.tiny.common.exception.Asserts;
import com.macro.mall.tiny.common.util.MyFileUtils;
import com.macro.mall.tiny.config.UploadConfig;
import com.macro.mall.tiny.modules.business.mapper.KrBLineMapper;
import com.macro.mall.tiny.modules.business.mapper.KrBRegionProvinceMapper;
import com.macro.mall.tiny.modules.business.model.BFile;
import com.macro.mall.tiny.modules.business.model.KrBLine;
import com.macro.mall.tiny.modules.business.model.KrBRegionProvince;
import com.macro.mall.tiny.modules.business.model.KrBTower;
import com.macro.mall.tiny.modules.business.service.KrBLineCacheService;
import com.macro.mall.tiny.modules.business.service.KrBLineService;
import com.macro.mall.tiny.security.util.ShiroAuthUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * <p>
 * 线路表 服务实现类
 * </p>
 *
 * @author macro
 * @since 2021-03-24
 */
@Service
public class KrBLineServiceImpl extends ServiceImpl<KrBLineMapper, KrBLine> implements KrBLineService {

    @Autowired
    private KrBLineCacheService krBLineCacheService;

    @Autowired
    private KrBLineMapper krBLineMapper;

    @Autowired
    KrBRegionProvinceMapper krBRegionProvinceMapper;

    @Override
    public KrBLine findById(Long id) {
        return krBLineMapper.findById(id);
    }

    @Override
    public List<KrBLine> getLineList() {
        List<KrBLine> lineList = krBLineCacheService.getLineList();
        if (CollUtil.isNotEmpty(lineList)) {
            return lineList;
        }
        lineList = krBLineMapper.findListMulti();
        if (CollUtil.isNotEmpty(lineList)) {
            krBLineCacheService.setLineList(lineList);
        } else {
            krBLineCacheService.setLineList(new ArrayList<>());
        }
        return lineList;
    }

    @Override
    public List<KrBLine> findByKey(String key, String provinceId, String cityId) {
        List<KrBLine> lineList = krBLineMapper.findListByKey(key, provinceId, cityId);
        if (StringUtils.isNotBlank(key)) {
            lineList.forEach(line -> {
                if (!line.getProvince().contains(key) && !line.getLineName().contains(key)) {
                    String keyLow = key.toLowerCase();
                    Set<String> matchShapeList = line.getTowerList().stream()
                            .filter(tower -> tower.getTowerNumber().toLowerCase().contains(keyLow))
                            .map(KrBTower::getTowerNumber).collect(Collectors.toSet());
                    line.setMatchShapeList(matchShapeList);
                }
            });
        }
        return lineList;
    }

    @Override
    public List<KrBLine> findByKeySimple(String key, String provinceId, String cityId) {
        List<KrBLine> lineList = krBLineMapper.findByKeySimple(key, provinceId, cityId);
        if (StringUtils.isNotBlank(key)) {
            lineList.forEach(line -> {
                if (!line.getProvince().contains(key) && !line.getLineName().contains(key)) {
                    String keyLow = key.toLowerCase();
                    Set<String> matchShapeList = line.getTowerList().stream()
                            .filter(tower -> tower.getTowerNumber().toLowerCase().contains(keyLow))
                            .map(KrBTower::getTowerNumber).collect(Collectors.toSet());
                    line.setMatchShapeList(matchShapeList);
                }
            });
        }
        return lineList;
    }

    @Override
    public List<BFile> getFile(Long id) {
        boolean auth = ShiroAuthUtil.checkAccess(ShiroAuthUtil.ACCESS);
        KrBLine line = krBLineMapper.selectById(id);
        if (line == null) Asserts.fail("该线路不存在，id：" + id);
        LambdaQueryWrapper<KrBRegionProvince> provinceWrapper = new QueryWrapper<KrBRegionProvince>().lambda().eq(KrBRegionProvince::getProvinceId, line.getProvinceId());
        KrBRegionProvince province = krBRegionProvinceMapper.selectOne(provinceWrapper);
        String basePath = UploadConfig.powerPath +
                province.getName() + UploadConfig.SEPARATOR +
                line.getLineName();
        return MyFileUtils.getSubBFiles(basePath, auth);
    }
}
