package com.macro.mall.tiny.modules.business.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.collect.Lists;
import com.macro.mall.tiny.common.exception.Asserts;
import com.macro.mall.tiny.common.util.MyFileUtils;
import com.macro.mall.tiny.config.UploadConfig;
import com.macro.mall.tiny.modules.business.enums.DirectoryEnum;
import com.macro.mall.tiny.modules.business.mapper.KrBLineMapper;
import com.macro.mall.tiny.modules.business.mapper.KrBRegionProvinceMapper;
import com.macro.mall.tiny.modules.business.mapper.KrBTowerMapper;
import com.macro.mall.tiny.modules.business.model.BFile;
import com.macro.mall.tiny.modules.business.model.KrBLine;
import com.macro.mall.tiny.modules.business.model.KrBRegionProvince;
import com.macro.mall.tiny.modules.business.model.KrBTower;
import com.macro.mall.tiny.modules.business.service.KrBTowerService;
import com.macro.mall.tiny.security.util.ShiroAuthUtil;
import org.apache.commons.lang3.StringUtils;
import org.apache.shiro.SecurityUtils;
import org.apache.shiro.subject.Subject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.io.FilenameFilter;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 塔表 服务实现类
 * </p>
 *
 * @author macro
 * @since 2021-03-24
 */
@Service
public class KrBTowerServiceImpl extends ServiceImpl<KrBTowerMapper, KrBTower> implements KrBTowerService {

    @Autowired
    KrBTowerMapper krBTowerMapper;

    @Autowired
    KrBLineMapper krBLineMapper;

    @Autowired
    KrBRegionProvinceMapper krBRegionProvinceMapper;

    @Override
    public long saveAndUpdate(List<KrBTower> krBTowerList) {
        return krBTowerMapper.saveAndUpdate(krBTowerList);
    }

    @Override
    public Map<String, List<BFile>> getFile(Long id) {
        boolean auth = ShiroAuthUtil.checkAccess(ShiroAuthUtil.ACCESS);

        HashMap<String, List<BFile>> resultMap = new HashMap<>();

        KrBTower tower = krBTowerMapper.selectById(id);
        if (tower == null) Asserts.fail("该电塔不存在，id：" + id);
        KrBLine line = krBLineMapper.selectById(tower.getLineId());
        if (line == null) Asserts.fail("该电塔所在的线路不存在，lineId：" + tower.getLineId());
        LambdaQueryWrapper<KrBRegionProvince> provinceWrapper = new QueryWrapper<KrBRegionProvince>().lambda().eq(KrBRegionProvince::getProvinceId, line.getProvinceId());
        KrBRegionProvince province = krBRegionProvinceMapper.selectOne(provinceWrapper);
        String basePath = UploadConfig.powerPath +
                province.getName() + UploadConfig.SEPARATOR +
                line.getLineName();

        String secBasePath;
        String thirdBasePath;
        List<BFile> sonList;
        // 获取3杆塔图纸
        String shape = tower.getTowerNumber();
        int lastIndex = shape.lastIndexOf('-');
        if (lastIndex > 0) {
            shape = shape.substring(0, lastIndex).replace('/', ' ');
        }
        if (StringUtils.isNotBlank(shape)) {
            secBasePath = MyFileUtils.getAuthFilePath(basePath, DirectoryEnum.TOWER_PIC.getName(), auth);
            thirdBasePath = MyFileUtils.getAuthFilePath(secBasePath, shape, auth);
            sonList = MyFileUtils.getSubBFiles(thirdBasePath, auth);
        } else {
            sonList = Lists.newArrayList();
        }
        resultMap.put("shapeFileList", sonList);

        // 获取6地线金具串图
        secBasePath = MyFileUtils.getAuthFilePath(basePath, DirectoryEnum.EARTH_WIRE_HARD_PIC.getName(), auth);
        String hardwareType = tower.getHardwareType().replace('/', ' ');
        if (StringUtils.isNotBlank(hardwareType)) {
            FilenameFilter hardwareTypeFilter = (dir, name) -> name.startsWith(hardwareType) || (name.startsWith(UploadConfig.AUTH_STR + hardwareType) && auth);
            sonList = MyFileUtils.getSubBFiles(secBasePath, hardwareTypeFilter, auth);
        } else {
            sonList = Lists.newArrayList();
        }
        resultMap.put("hardwareFileList", sonList);

        // 获取7班组巡检照片
        secBasePath = MyFileUtils.getAuthFilePath(basePath, DirectoryEnum.TEAM_CHECK_PIC.getName(), auth);
        String towerName = tower.getTowerName();
        thirdBasePath = MyFileUtils.getAuthFilePath(secBasePath, towerName, auth);
        sonList = MyFileUtils.getSubBFiles(thirdBasePath, auth);
        resultMap.put("teamCheckFileList", sonList);

        // 获取12机器巡检照片
        /*secBasePath = MyFileUtils.getAuthFilePath(basePath, DirectoryEnum.ROBOT_CHECK_PIC.getName(), auth);
        towerName = tower.getTowerName();
        thirdBasePath = MyFileUtils.getAuthFilePath(secBasePath, towerName, auth);
        sonList = MyFileUtils.getSubBFiles(thirdBasePath, auth);
        resultMap.put("robotCheckFileList", sonList);*/

        return resultMap;
    }

    @Override
    public void setHasFile(List<KrBTower> towerList) {
        Subject subject = SecurityUtils.getSubject();
        setHasFile(towerList, subject);
    }

    @Override
    @Transactional(propagation = Propagation.REQUIRED)
    public void setHasFile(List<KrBTower> towerList, Subject subject) {
        if (towerList == null || towerList.size() == 0) {
            return;
        }
        boolean access = ShiroAuthUtil.checkAccess(subject, ShiroAuthUtil.ACCESS);
        KrBTower firstTower = towerList.get(0);
        KrBLine line = krBLineMapper.selectById(firstTower.getLineId());
        LambdaQueryWrapper<KrBRegionProvince> provinceWrapper = new QueryWrapper<KrBRegionProvince>().lambda().eq(KrBRegionProvince::getProvinceId, line.getProvinceId());
        KrBRegionProvince province = krBRegionProvinceMapper.selectOne(provinceWrapper);
        String basePath = UploadConfig.powerPath +
                province.getName() + UploadConfig.SEPARATOR +
                line.getLineName();

        for (KrBTower tower : towerList) {
            String secBasePath;
            String thirdBasePath;

            // 检验是否包含塔形资料
            String shape = tower.getTowerNumber();
            if (StringUtils.isNotBlank(shape)) {
                int lastIndex = shape.lastIndexOf('-');
                if (lastIndex > 0) {
                    shape = shape.substring(0, lastIndex).replace('/', ' ');
                }
                if (StringUtils.isNotBlank(shape)) {
                    secBasePath = MyFileUtils.getAuthFilePath(basePath, DirectoryEnum.TOWER_PIC.getName(), access);
                    thirdBasePath = MyFileUtils.getAuthFilePath(secBasePath, shape, access);
                    if (MyFileUtils.hasSonFile(thirdBasePath, access)) {
                        tower.setHasFile(1);
                        continue;
                    }
                }
            }

            // 检验是否含有金具串资料
            String hardwareType = tower.getHardwareType();
            if (StringUtils.isNotBlank(hardwareType)) {
                secBasePath = MyFileUtils.getAuthFilePath(basePath, DirectoryEnum.EARTH_WIRE_HARD_PIC.getName(), access);
                String hardwareTypeReplace = hardwareType.replace('/', ' ');
                FilenameFilter hardwareTypeFilter = (dir, name) -> name.startsWith(hardwareTypeReplace) || (name.startsWith(UploadConfig.AUTH_STR + hardwareTypeReplace) && access);
                if (MyFileUtils.hasSonFile(secBasePath, hardwareTypeFilter, access)) {
                    tower.setHasFile(1);
                    continue;
                }
            }

            // 检验是否含有巡检照片
            secBasePath = MyFileUtils.getAuthFilePath(basePath, DirectoryEnum.TEAM_CHECK_PIC.getName(), access);
            thirdBasePath = MyFileUtils.getAuthFilePath(secBasePath, tower.getTowerName(), access);
            if (MyFileUtils.hasSonFile(thirdBasePath, access)) {
                tower.setHasFile(1);
                continue;
            }

            // 检验是否含有机器巡检照片
            /*secBasePath = MyFileUtils.getAuthFilePath(basePath, DirectoryEnum.ROBOT_CHECK_PIC.getName(), access);
            thirdBasePath = MyFileUtils.getAuthFilePath(secBasePath, tower.getTowerName(), access);
            if (MyFileUtils.hasSonFile(thirdBasePath, access)) {
                tower.setHasFile(1);
                continue;
            }*/


            tower.setHasFile(0);
        }

        krBTowerMapper.saveAndUpdate(towerList);
    }

}
